package shef.dcs.cashews.ui;

import java.io.InputStream;
import java.util.EventObject;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditor;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import shef.dcs.cashews.model.Diagram;
import shef.dcs.cashews.parts.PartFactory;
import shef.dcs.cashews.parts.TreePartFactory;


// We'll start off subclassing from GraphicalEditor, even though it is only designed as a sample.
// Later, we can directly extend EditorPart
public class DiagramEditor extends GraphicalEditor {

   private boolean savePreviouslyNeeded;

   private Diagram diagram = new Diagram();

   public DiagramEditor() {
      // an EditDomain is a "session" of editing which contains things
      // like the CommandStack
      setEditDomain(new DefaultEditDomain(this));
   }

   public Diagram getDiagram() {
      return this.diagram;
   }

   //------------------------------------------------------------------------
   // Overridden from GraphicalEditor

   protected void configureGraphicalViewer() {
      super.configureGraphicalViewer();
      getGraphicalViewer().setRootEditPart(new ScalableFreeformRootEditPart());

      // set the factory to use for creating EditParts for elements in the model
      getGraphicalViewer().setEditPartFactory(new PartFactory());
   }

   public void commandStackChanged(EventObject event) {
      if (isDirty()) {
         if (!this.savePreviouslyNeeded) {
            this.savePreviouslyNeeded = true;
            firePropertyChange(IEditorPart.PROP_DIRTY);
         }
      }
      else {
         savePreviouslyNeeded = false;
         firePropertyChange(IEditorPart.PROP_DIRTY);
      }
      super.commandStackChanged(event);
   }

   public Object getAdapter(Class type){
      if (type == IContentOutlinePage.class)
         return new OutlinePage();
      return super.getAdapter(type);
   }

   //------------------------------------------------------------------------
   // Overridden from EditorPart

   protected void setInput(IEditorInput input) {
      super.setInput(input);

      IFile file = ((IFileEditorInput) input).getFile();
      try { // attempt to read from a file
         InputStream istream = file.getContents(false);
         diagram = Diagram.makeFromStream(istream);
      }
      catch (Exception e) { // but if there's an error, create a new diagram
         e.printStackTrace();
         diagram = new Diagram();
      }
   }


   //------------------------------------------------------------------------
   // Abstract methods from GraphicalEditor

   protected void initializeGraphicalViewer() {
      // this uses the PartFactory set in configureGraphicalViewer
      // to create an EditPart for the diagram and sets it as the
      // content for the viewer
      getGraphicalViewer().setContents(this.diagram);
   }


   //------------------------------------------------------------------------
   // Abstract methods from EditorPart

   public void doSave(IProgressMonitor monitor) {
      try {
         IFile file = ((IFileEditorInput) getEditorInput()).getFile();
         file.setContents(this.diagram.getAsStream(), true, false, monitor);
         getCommandStack().markSaveLocation();
      }
      catch (Exception e) {
         e.printStackTrace();
      }
   }

   public void doSaveAs() {
      SaveAsDialog dialog = new SaveAsDialog(getSite().getWorkbenchWindow().getShell());
      dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
      dialog.open();
      IPath path = dialog.getResult();

      if (path == null)
         return;

      IWorkspace workspace = ResourcesPlugin.getWorkspace();
      final IFile file = workspace.getRoot().getFile(path);

      WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
         public void execute(final IProgressMonitor monitor) throws CoreException {
            try {
               file.create(diagram.getAsStream(), true, monitor);
            }
            catch (Exception e) {
               e.printStackTrace();
            }
         }
      };

      try {
         new ProgressMonitorDialog(getSite().getWorkbenchWindow().getShell()).run(false, true, op);
         setInput(new FileEditorInput((IFile) file));
         getCommandStack().markSaveLocation();
      }
      catch (Exception e) {
         e.printStackTrace();
      }
   }

   public void gotoMarker(IMarker marker) {
   }

   public boolean isDirty() {
      // rely on the command stack to determine dirty flag
      return getCommandStack().isDirty();
   }

   public boolean isSaveAsAllowed() {
      // allow Save As
      return true;
   }


   //========================================================================
   // Inner Classes

   public class OutlinePage extends ContentOutlinePage {

      private PageBook pageBook;
      private Control outline;

      public OutlinePage() {
         super(new TreeViewer());
      }

      public void createControl(Composite parent) {
         pageBook = new PageBook(parent, SWT.NONE);
         outline = getViewer().createControl(pageBook);
         pageBook.showPage(outline);
         getSelectionSynchronizer().addViewer(getViewer());
         getViewer().setEditDomain(getEditDomain());
         getViewer().setEditPartFactory(new TreePartFactory());
         getViewer().setContents(getDiagram());
      }

      public Control getControl() {
         return this.pageBook;
      }

      public void dispose() {
         getSelectionSynchronizer().removeViewer(getViewer());
         super.dispose();
      }
   }
}